# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
# - Alexis Sukrieh
# - Jean-Sebastien Pedron
package FDN::Web::Forms;

use strict;
use warnings;

use vars qw($VERSION);
$VERSION = '1.2.4';

use FDN::Common::Config;
use FDN::Common::Logger;
use FDN::Web::TemplateParse qw(remplace parse);

require Exporter;
our @ISA = qw(Exporter);

our @EXPORT = qw(
  &form_read
  &form_print_parts
  
  &form_keep_val 
  &form_check_val
  &form_set_field_value

  $FORM_PART_PATH
  $FORM_PART_XHTML
  $FORM_PART_TABLES
  $FORM_PART_TEMPLATE
  $FORM_PART_JAVASCRIPT
  
  $DEBUG
);

our @EXPORT_OK = qw(
  &form_new_selval
  &form_add_selval
  &form_del_selval
  
  &form_new_field
  &form_add_field
  &form_del_field

  &form_new_block
  &form_add_block
  &form_del_block

  &form_new_part
  &form_add_part
  &form_del_part

  &form_new

  &str_http_encode
  &str_http_decode
  &str_js_escape
  
  &form_read_select
  &form_read_text
  &form_read_password
  &form_read_file
  &form_read_textarea
  &form_read_hidden
  &form_read_image
  &form_read_resetimage
  &form_read_checkbox
  &form_read_button
  &form_read_submit
  &form_read_reset
  &form_read_html
  &form_read_emptycell
  
  &form_set_select_value
  &form_set_checkbox_value
  &form_set_radio_value
  &form_set_text_value
  &form_set_textarea_value
  &form_set_html_value
  &form_set_button_value
  &form_set_hidden_value
);

our %EXPORT_TAGS = (
  'dynamics' => [ @EXPORT, qw(
    form_new_selval form_add_selval form_del_selval
    form_new_field  form_add_field  form_del_field
    form_new_block  form_add_block  form_del_block
    form_new_part   form_add_part   form_del_part
    form_new
  ) ],
  'tools'    => [ @EXPORT, qw(str_http_encode str_http_decode str_js_escape) ],
  'ALL'      => [ @EXPORT, @EXPORT_OK ]
);

# --------------------------------------------------------------------
# Constantes et variables globales
# --------------------------------------------------------------------

# Nom du template utilis pour rendre les form-parts
our $FORM_PART_PATH       = '/usr/share/fdn/templates/forms';
our $FORM_PART_XHTML      = 'form-part.html';
our $FORM_PART_TABLES     = 'form-part-tables.html';
our $FORM_PART_JAVASCRIPT = 'form-part.js';

# Le template par dfaut pour form_print_parts()
our $FORM_PART_TEMPLATE   = $FORM_PART_TABLES;

# Mode de debugging (off par dfaut)
our $DEBUG = 0;

# Cette variable garde une rfrence au formulaire (la mme que celle
# retourne par form_read(). Elle sert pour form_set_checkbox_value()
# et form_set_radio_value() qui ont besoin de retrouver tous les inputs
# qui correspondent  un mme nom (nom au sens HTML).
my $reading_form;

# Cette variable contient le prochain tabindex (attribut HTML) disponible.
my $tabindex = 1;

# Ce template permet la gnration de la fonction JavaScript
# check_all_parts().
my $check_all_parts_template = << 'EOF';
var check_all_parts_defined = 1;

function check_all_parts() {
  var parts_ok = 1;
  %%LOOP-parts-%%
  if (typeof(part_%%name%%_defined) != "undefined" && part_%%name%%_defined) {
    parts_ok = parts_ok && check_part_%%name%%();
  }
  %%-parts-ELOOP%%
  return parts_ok;
}
EOF

my $check_all_parts_call = 'javascript:return check_all_parts();';

# --------------------------------------------------------------------
# Fonctions principales
# --------------------------------------------------------------------

#
# Lit un fichier de configuration et retourne une structure prte pour
# form_print_parts().
#
sub form_read ($$) {
  my ($file, $name) = @_;
  
  debug "Utilisation du fichier de config `$file'" if $DEBUG;
  my $config = Load_Config($file);
  
  my $form = form_new($config, $name);
  
  return $form;
}

#
# Prend une structure retourne par form_read() et un nom de template
# pour gnrer une page HTML contenant le(s) formulaire(s).
# 
sub form_print_parts ($;$) {
  my ($form, $file) = @_;
  return unless (defined $form);
  
  $file = $FORM_PART_PATH.'/'.$FORM_PART_TEMPLATE unless $file;
  
  my $glob_form = {};
  my $check_all_parts = parse($check_all_parts_template, $form);
  
  $glob_form->{'form_name'}       = $form->{'form_name'};
  $glob_form->{'form_action'}     = $form->{'form_action'};
  $glob_form->{'form_method'}     = $form->{'form_method'};
  
  $glob_form->{'FORM_PART_PATH'}       = $FORM_PART_PATH;
  $glob_form->{'FORM_PART_TEMPLATE'}   = $FORM_PART_TEMPLATE;
  $glob_form->{'FORM_PART_JAVASCRIPT'} = $FORM_PART_JAVASCRIPT;
  
  # Les paramtres en rapport avec check_all_part()
  $glob_form->{'check_all_parts'} = $check_all_parts;
  $glob_form->{'form_onsubmit'}   =
    $form->{'onsubmit'} || $check_all_parts_call;
  
  for my $pr (@{$form->{'parts'}}) {
    $pr->{'mandatory_radios'}     = $form->{'mandatory_radios'};
    $pr->{'mandatory_checkboxes'} = $form->{'mandatory_checkboxes'};
    $glob_form->{'FORMULAIRE_'.$pr->{'name'}} = remplace($file, $pr, $pr);
  }
  
  # Le prochain tabindex
  $glob_form->{'next_tabindex'} = $form->{'next_tabindex'};
  
  return $glob_form;
}

#
# Met  jour la valeurs des champs du formulaire (dans la structure retourne
# par form_read())  partir des paramtres de l'URL.
#
sub form_keep_val ($$) {
  my ($form, $values) = @_;
  
  # Pour chaque paramtre de l'URL, on affecte la bonne valeur au champ
  # correspondant
  for my $name (keys %$values) {
    if ($form->{'byhtmlname'}->{$name}[0]->{'keep'}) {
      #info "Keeping value of $name";
      form_set_field_value($form->{'byhtmlname'}->{$name}[0], $values->{$name});
    } else {
      debug "Le champ `$name' n'a pas l'attribut `keep'  vrai,".
        "sa valeur n'est pas retenue" if ($DEBUG);
    }
  }
}

#
# Procde aux vrifications des valeurs des champs, comme le fait le code
# JavaScript gnr. Cette fonction est utile car le JavaScript n'est pas
# fiable (l'utilisateur peut le dsactiver ou le navigateur ne le supporte
# peut-tre pas).
#
sub form_check_val ($$;$) {
  my ($form, $values, $report) = @_;
  my $ret = 1;
  my $name;
  
  # On vrifie qu'on nous a donn 3 paramtres (c--d, le $report est
  # bien l)
  my $has_report = (scalar @_ >= 3) ? 1 : 0;
  
  # Les champs obligatoires sont-ils renseigns ?
  for $name (keys %{$form->{'byhtmlname'}}) {
    my $field = $form->{'byhtmlname'}->{$name}[0];
    
    if ($field->{'mandatory'}) {
      if (!defined $values->{$name} || $values->{$name} eq '') {
        debug "Le champ `$name' n'est pas renseign" if ($DEBUG);
        $ret = 0;
        
        if ($has_report) {
          $report->{'mandatory'}->{$name} = $field->{'mandatory_error'};
        }
      }
    }
  }
  
  # Les champs renseigns sont-ils valides ?
  for $name (keys %$values) {
    my $field = $form->{'byhtmlname'}->{$name}[0];
    
    # Vrification par regexp
    if ($field->{'regexp'}) {
      if (!$field->{'mandatory'} && $values->{$name} eq '') {
        debug "Le champ `$name' est accept (car non obligatoire)"
          if ($DEBUG);
        next;
      }
      
      my ($re, $flags) = $field->{'regexp'} =~ /^\/(.*)\/([^\/]*)$/o;
      $re = $flags ? eval("qr/$re/$flags") : qr/$re/;
      if ($values->{$name} !~ /$re/) {
        debug "Le champ `$name' est rejet; valeur = ".$values->{$name}
          if ($DEBUG);
        $ret = 0;
        
        if ($has_report && !exists $report->{'mandatory'}->{$name}) {
          $report->{'invalid'}->{$name} = $field->{'regexp_error'};
        }
      }
    }
    
    # Vrification par rejected
    if ($field->{'rejected'}) {
      if (!$field->{'mandatory'} && $values->{$name} eq '') {
        debug "Le champ `$name' est accept (car non obligatoire)"
          if ($DEBUG);
        next;
      }
      
      my ($re, $flags) = $field->{'rejected'} =~ /^\/(.*)\/([^\/]*)$/o;
      $re = $flags ? eval("qr/$re/$flags") : qr/$re/;
      if ($values->{$name} =~ /$re/) {
        debug "Le champ `$name' est rejet; valeur = ".$values->{$name}
          if ($DEBUG);
        $ret = 0;
        
        if ($has_report && !exists $report->{'mandatory'}->{$name}) {
          $report->{'invalid'}->{$name} = $field->{'rejected_error'};
        }
      }
    }
    
    # Vrification avec le module FDN::Common::Validate
    if ($field->{'validate'}) {
      if (!$field->{'mandatory'} && $values->{$name} eq '') {
        debug "Le champ `$name' est accept (car non obligatoire)"
          if ($DEBUG);
        next;
      }
      
      my $fct  = $field->{'validate'};
      my $code = "use FDN::Common::Validate; $fct('".
      $values->{$name}."');";
      unless (eval($code)) {
        if ($@) {
          error "Le champ `$name' a fait appel  `$fct', mais : $@\n.".
          "Le code valu est '$code'";
        } else {
          debug "Le champ `$name' est rejet; valeur = ".$values->{$name}
            if ($DEBUG);
        }
        $ret = 0;
        
        if ($has_report && !exists $report->{'mandatory'}->{$name} &&
          !exists $report->{'invalid'}->{$name}) {
          $report->{'invalid'}->{$name} = $field->{'validate_error'};
        }
      }
    }
  }
  
  return $ret;
}

# --------------------------------------------------------------------
# Fonctions de gestion dynamique d'un formulaire
# --------------------------------------------------------------------

#
# Cr une nouvelle option pour l'ajouter  un champ `select'.
#
sub form_new_selval ($$);
sub form_new_selval ($$) {
  my ($config, $name) = @_;
  return unless defined($name);
  __check_name($name) if ($DEBUG);
  
  my $val = $config->{'selval-'.$name};
  
  debug "Lecture de la selval `$name'" if ($DEBUG);
  
  unless (defined $val) {
    error "La valeur `selval-$name' est utilise mais n'est pas definie";
    return;
  }
  
  my $res = { 'selval_name' => $name };
  
  # Est-ce un optgroup ou une option ?
  if (defined $val->{'@values'}) {
    # Optgroup
    $res->{'optgroup'} = 1;
    $res->{'options'} = [];
    
    for my $sv (@{$val->{'@values'}}) {
      debug "L'optgroup `$name' contient la valeur `$sv'" if ($DEBUG);
      my $opt = form_new_selval($config, $sv);
      unless (defined $opt) {
        error "La selval `$sv' de l'optgroup `$name' est inconnue";
        next;
      }
      
      push @{$res->{'options'}}, $opt;
    }
  } else {
    # Option
    if (defined $val->{'value'}) {
      $res->{'option_value'} = $val->{'value'};
    } else {
      info "La valeur `selval-$name' n'a pas d'attribut `value'";
      $res->{'option_value'} = $name;
    }  
  }
  
  # Le label
  if (defined $val->{'label'}) {
    $res->{'option_label'} = $val->{'label'};
  } else {
    if ($res->{'optgroup'}) {
      $res->{'option_label'} = ucfirst($name);
    } else {
      $res->{'option_label'} = $res->{'option_value'};
    }
  }
  
  # Est-elle dsactive ?
  if (defined $val->{'disabled'} && $val->{'disabled'} eq 'true') {
    $res->{'disabled'} = 1;
  }
  
  # L'id
  if ($val->{'id'}) {
    $res->{'option_id'} = $val->{'id'};
  }
  
  return $res;
}

#
# Cr un champ  partir des informations trouves dans la config
# (passe sous forme de refrnce  un hash), en fonction d'un nom. Ce
# champ n'est attach  aucun bloc.
#
sub form_new_field ($$) {
  my ($config, $name) = @_;
  return unless defined($name);
  __check_name($name) if ($DEBUG);
  
  my $field = $config->{'field-'.$name};
  
  debug "Lecture du champ `$name'" if ($DEBUG);
  
  if (!defined $field || ref($field) ne 'HASH') {
    error "Champ `$name' non defini";
    return;
  }
  
  my $res = { 'field_name' => $name };
  my $add = {};
  
  # Le nom : soit l'attribut `nom', soit l'identifiant de la section
  if (defined $field->{'name'}) {
    $res->{'input_name'} = $field->{'name'};
  } else {
    $field->{'name'}     = $name;
    $res->{'input_name'} = $name;
  }
  
  # Est-ce un champ dsactiv ?
  if (defined $field->{'disabled'} and $field->{'disabled'} eq 'true') {
    $res->{'disabled'} = 1;
  }
  
  # Ce champ s'tend t'il sur plusieurs colonnes ?
  if (defined $field->{'colspan'} and
      $field->{'colspan'}) {
    $res->{'colspan'} = $field->{'colspan'};
  }
  
  # Est-il obligatoire ?
  # Note : on ne vrifie plus si mandatory et disabled sont en conflits
  if (defined $field->{'mandatory'}) {
    if ($field->{'mandatory'} eq 'true') {
      $res->{'mandatory'} = 1;
    } else {
      $res->{'mandatory'} = 0;
    }
  }
  
  # L'id
  if (defined $field->{'id'}) {
    $res->{'input_id'} = $field->{'id'};
  } else {
    $res->{'input_id'} = $res->{'input_name'};
  }
  
  # Le tabindex
  unless ($res->{'disabled'} ||
          $field->{'type'} eq 'emptycell' ||
          $field->{'type'} eq 'hidden' ||
          $field->{'type'} eq 'html') {
    $res->{'tabindex'} = $tabindex++;
  }

  # L'accesskey
  if ($field->{'accesskey'} && !$res->{'disabled'}) {
    $res->{'accesskey'} = $field->{'accesskey'};
  }
  
  # Le label
  if (defined $field->{'label'}) {
    $res->{'input_label'} = $field->{'label'};
  }
  
  # Le message pour les champs obligatoires non-saisis
  if (defined $field->{'mandatory_error'}) {
    $res->{'mandatory_error'} = $field->{'mandatory_error'};
  } elsif (defined $field->{'label_erreur'}) {
    # XXX Backward compatibility
    warning "L'attribut `label_erreur' du champ `$name' est obsolete. Veuillez utiliser `mandatory_error'  la place";
    $res->{'mandatory_error'} = $field->{'label_erreur'};
  } elsif (defined $res->{'input_label'}) {
    $res->{'mandatory_error'} = "Le champ  ".$res->{'input_label'}.
      "  n'est pas renseign";
  } else {
    $res->{'mandatory_error'} = "Le champ  ".$res->{'input_name'}.
      "  n'est pas renseign";
  }
  $res->{'mandatory_error'} = str_js_escape($res->{'mandatory_error'});
  
  # Le keep
  if (defined $field->{'keep'} && $field->{'keep'} ne 'true') {
    $res->{'keep'} = 0;
  } else {
    $res->{'keep'} = 1;
  }
  
  # La regexp
  # Note : on ne vrifie plus si mandatory et disabled sont en conflits
  if (defined $field->{'regexp'}) {
    $res->{'regexp'} = $field->{'regexp'};
    unless ($res->{'regexp'} =~ /^\/.*\/[^\/]*$/o) {
      error "La regexp du champ `$name' ne contient pas de slashs (/) au dbut et  la fin (hors options)";
    }
  }
  
  # Le message pour les erreurs de saisie
  if (defined $field->{'regexp_error'}) {
    $res->{'regexp_error'} = $field->{'regexp_error'};
  } elsif (defined $field->{'label_syntaxe'}) {
    # XXX Backward compatibility
    warning "L'attribut `label_syntaxe' du champ `$name' est obsolete. Veuillez utiliser `regexp_error'  la place";
    $res->{'regexp_error'} = $field->{'label_syntaxe'};
  } elsif (defined $res->{'input_label'}) {
    $res->{'regexp_error'} = "Le champ  ".$res->{'input_label'}.
      "  n'est pas correct";
  } else {
    $res->{'regexp_error'} = "Le champ  ".$res->{'input_name'}.
      "  n'est pas correct";
  }
  $res->{'regexp_error'} =~ str_js_escape($res->{'regexp_error'});
  
  # La regex des valeurs refuses
  if (defined $field->{'rejected'}) {
    $res->{'rejected'} = $field->{'rejected'};
    unless ($res->{'rejected'} =~ /^\/.*\/[^\/]*$/o) {
      error "L'attribut rejected du champ `$name' ne contient pas de slashs (/) au dbut et  la fin (hors options)";
    }
  }
  
  # Le message pour les valeurs refuses
  if (defined $field->{'rejected_error'}) {
    $res->{'rejected_error'} = $field->{'rejected_error'};
  } elsif (defined $res->{'input_label'}) {
    $res->{'rejected_error'} = "Le champ  ".$res->{'input_label'}.
      "  n'est pas correct";
  } else {
    $res->{'rejected_error'} = "Le champ  ".$res->{'input_name'}.
      "  n'est pas correct";
  }
  $res->{'rejected_error'} =~ str_js_escape($res->{'rejected_error'});
  
  # Validate (fonction de FDN::Common::Validate)
  # Note : on ne vrifie plus si mandatory et disabled sont en conflits
  if (defined $field->{'validate'}) {
    $res->{'validate'} = $field->{'validate'};
  }
  
  # Le message pour les erreurs de saisie
  if (defined $field->{'validate_error'}) {
    $res->{'validate_error'} = $field->{'validate_error'};
  } else {
    $res->{'validate_error'} = $res->{'regexp_error'};
  }
  $res->{'validate_error'} =~ str_js_escape($res->{'validate_error'});
  
  # Les infos specifiques au type de champ
  $res->{'input_type'} = $field->{'type'};
  if ($field->{'type'} eq 'select') {
    $add = form_read_select($config, $field);
  } elsif ($field->{'type'} eq 'text') {
    $add = form_read_text($config, $field);
  } elsif ($field->{'type'} eq 'password') {
    $add = form_read_password($config, $field);
  } elsif ($field->{'type'} eq 'file') {
    $add = form_read_file($config, $field);
  } elsif ($field->{'type'} eq 'textarea') {
    $add = form_read_textarea($config, $field);
  } elsif ($field->{'type'} eq 'hidden') {
    $add = form_read_hidden($config, $field);
  } elsif ($field->{'type'} eq 'image') {
    $add = form_read_image($config, $field);
  } elsif ($field->{'type'} eq 'resetimage') {
    $add = form_read_resetimage($config, $field);
  } elsif ($field->{'type'} eq 'checkbox') {
    $add = form_read_checkbox($config, $field);
  } elsif ($field->{'type'} eq 'emptycell') {
    $add = form_read_emptycell($config, $field);
  } elsif ($field->{'type'} eq 'html') {
    $add = form_read_html($config, $field);
  } elsif ($field->{'type'} eq 'radio') {
    $add = form_read_radio($config, $field);
  } elsif ($field->{'type'} eq 'button') {
    $add = form_read_button($config, $field);
  } elsif ($field->{'type'} eq 'submit') {
    $add = form_read_submit($config, $field);
  } elsif ($field->{'type'} eq 'reset') {
    $add = form_read_reset($config, $field);
  } else {
    error 'Type de champ inconnu';
  }
  
  $res = { %$res, %$add };
  return $res;
}

#
# Cr un bloc  partir des informations trouves dans la config (passe
# sous forme de rfrence  un hash), en fonction d'un nom. Ce bloc
# n'est attach  aucune partie.
#
sub form_new_block ($$) {
  my ($config, $name) = @_;
  return unless defined($name);
  __check_name($name) if ($DEBUG);
  
  my $block = $config->{'block-'.$name};
  
  debug "Lecture du bloc `$name'" if ($DEBUG);
  
  if (!defined $block || ref($block) ne 'HASH') {
    error "Bloc de formulaire `$name' inconnu";
    return;
  }
  
  my $res = { 'block_name' => $name };
  
  # Le titre
  if (defined $block->{'title'}) {
    $res->{'block_title'} = $block->{'title'};
  }
  
  # L'id
  if (defined $block->{'id'}) {
    $res->{'block_id'} = $block->{'id'};
  } else {
    $res->{'block_id'} = $name;
  }
  
  if (defined $block->{'type'}) {
    $res->{'block_type'} = $block->{'type'};
  } else {
    $res->{'block_type'} = 'form-elem';
  }
  
  if ($res->{'block_type'} eq 'hidden-elem') {
    $res->{'block_is_hidden'} = 1;
  } else {
    $res->{'block_is_hidden'} = 0;
  }
  
  $res->{'block_elems'} = [];
  $res->{'byid'}        = {};
  $res->{'byname'}      = {};
  $res->{'byhtmlname'}  = {};
  
  if (defined $block->{'@fields'} && ref($block->{'@fields'}) eq 'ARRAY') {
    for my $fl (@{$block->{'@fields'}}) {
      debug "Le bloc `$name' contient le champ `$fl'" if ($DEBUG);
      
      my $field = form_new_field($config, $fl);
      unless (defined $field) {
        error "Le champ `$fl' du bloc `$name' n'est pas defini";
        next;
      }
      
      $res->{'byname'}->{$fl} = $field;
      $res->{'byid'}->{$field->{'input_id'}} = $field;
      push @{$res->{'byhtmlname'}->{$field->{'input_name'}}}, $field;
      
      $field->{'block_type'}      = $res->{'block_type'};
      $field->{'block_is_hidden'} = $res->{'block_is_hidden'};
      $field->{'parent_block'}    = $res;
      push @{$res->{'block_elems'}}, $field;
    }
  }
  
  return $res;
}

#
# Cr une partie de formulaire  partir des informations trouves dans
# la config (passe sous forme de rfrence  un hash), en fonction d'un
# nom. Cette partie n'est attache  aucun formulaire.
#
sub form_new_part ($$) {
  my ($config, $name) = @_;
  return unless defined($name);
  __check_name($name) if ($DEBUG);
  
  my $part = $config->{'form-'.$name};
  
  debug "Lecture de la partie `$name'" if ($DEBUG);
  
  if (!defined $part || ref($part) ne 'HASH') {
    error "Partie $name inconnue";
    return;
  }
  
  my $res = { 'part_name' => $name };
  
  # L'id
  if (defined $part->{'id'}) {
    $res->{'part_id'} = $part->{'id'};
  } else {
    $res->{'part_id'} = "formulaire-$name";
  }
  
  # Les blocs
  $res->{'name'}         = $name;
  $res->{'blocks'}       = [];
  $res->{'blocksbyname'} = {};
  $res->{'byid'}         = {};
  $res->{'byname'}       = {};
  $res->{'byhtmlname'}   = {};
  
  # Tabindex
  if ($part->{'tabindex_start'}) {
    $res->{'tabindex_start'} = $part->{'tabindex_start'};
    if ($part->{'tabindex_start'} >= $tabindex) {
      $tabindex = $part->{'tabindex_start'};
    } else {
      warning "La partie `$name' indique un tabindex (".
        $part->{'tabindex_start'}.
        ") trop petit par rapport au prochain disponible ($tabindex)";
    }
  } elsif ($part->{'tabindex_offset'}) {
    $res->{'tabindex_offset'} = $part->{'tabindex_offset'};
    $tabindex += $part->{'tabindex_offset'};
  }
  
  # Les blocs appartenant  cette partie
  for my $bl (@{$part->{'@blocks'}}) {
    debug "La partie `$name' contient le bloc `$bl'" if ($DEBUG);
    
    my $block = form_new_block($config, $bl);
    if (!defined($block)) {
      error "Le bloc `$bl' de la partie `$name' est inconnu";
      next;
    }
    
    # Tables de correspondance pour les champs
    $res->{'byid'} = {
      %{$block->{'byid'}},
      %{$res->{'byid'}}
    };
    $res->{'byname'} = {
      %{$block->{'byname'}},
      %{$res->{'byname'}}
    };
    
    for my $n (keys %{$block->{'byhtmlname'}}) {
      push @{$res->{'byhtmlname'}->{$n}}, @{$block->{'byhtmlname'}->{$n}};
    }
    
    # Table de correspondance pour les blocs
    $res->{'blocksbyname'}->{$bl} = $block;
    
    $block->{'parent_part'} = $res;
    push @{$res->{'blocks'}}, $block;
  }
  
  return $res;
}

#
# Cr un formulaire
#
sub form_new ($$) {
  my ($config, $name) = @_;
  return unless defined($name);
  __check_name($name) if ($DEBUG);
  
  my $form = $config->{'formglob-'.$name};
  
  debug "Lecture du formulaire `$name'" if ($DEBUG);
  
  if (!defined $form || ref($form) ne 'HASH') {
    error "Formulaire `$name' inconnu";
    return;
  }
  
  my $res = { 'formglob_name' => $name };
  $reading_form = $res;
  
  $res->{'parts'}        = [];
  $res->{'partsbyname'}  = {};
  $res->{'blocksbyname'} = {};
  $res->{'byid'}         = {};
  $res->{'byname'}       = {};
  $res->{'byhtmlname'}   = {};
  
  # Le nom
  if (defined $form->{'name'}) {
    $res->{'form_name'} = $form->{'name'};
  } else {
    $res->{'form_name'} = $name;
  }
  
  # La methode
  if (defined $form->{'method'}) {
    $res->{'form_method'} = $form->{'method'};
  } else {
    $res->{'form_method'} = 'POST';
  }
  
  # L'action
  if (defined $form->{'action'}) {
    $res->{'form_action'} = $form->{'action'};
  } else {
    warning "Le formulaire `$name' n'a pas d'action, il est cependant obligatoire en HTML";
  }
  
  # Le template
  if (defined $form->{'template'}) {
    $res->{'template'} = $form->{'template'};
  } else {
    info "Le formulaire `$name' n'a pas de template";
  }
  
  # La merde de la BU
  if (defined $form->{'onsubmit'}) {
    $res->{'onsubmit'} = $form->{'onsubmit'};
  }
  
  # Les form-parts
  my @formparts = ();
  if (exists $form->{'@formparts'}) {
    @formparts = @{$form->{'@formparts'}};
  } elsif (exists $form->{'@formpart'}) {
    @formparts = @{$form->{'@formpart'}};
    warning "L'attribut `\@formpart' est OBSOLETE; ".
    "il doit tre remplac par `\@formparts' (pour tre cohrent avec ".
    "`\@blocks' et `\@fields'.";
  }
  
  for my $pr (@formparts) {
    debug "Le formulaire `$name' contient la partie `$pr'" if ($DEBUG);
    
    my $part = form_new_part($config, $pr);
    if (!defined($part)) {
      error "La partie `$pr' du formulaire `$name' est inconnue";
      next;
    }
    
    # Tables de correspondance pour les champs
    $res->{'byid'} = {
      %{$part->{'byid'}},
      %{$res->{'byid'}}
    };
    $res->{'byname'} = {
      %{$part->{'byname'}},
      %{$res->{'byname'}}
    };
    
    for my $n (keys %{$part->{'byhtmlname'}}) {
      push @{$res->{'byhtmlname'}->{$n}}, @{$part->{'byhtmlname'}->{$n}};
    }
    
    # Table de correspondance pour les blocs
    $res->{'blocksbyname'} = {
      %{$part->{'blocksbyname'}},
      %{$res->{'blocksbyname'}}
    };
    
    # Table de correspondance pour les parties
    $res->{'partsbyname'}->{$pr} = $part;
    
    $part->{'parent_form'} = $res;
    push @{$res->{'parts'}}, $part;
  }
  
  # On fournit  l'appeleur le prochain tabindex disponible
  $res->{'next_tabindex'} = $tabindex;
  
  # Si un champ comporte en ralit plusieurs champs (comme les radios),
  # certains attributs sont prciss dans un seul champ pour tout le groupe.
  # Ici, on propage ces attributs  tout le groupe. On prpare galement
  # 2 listes contenant les radios et checkboxes obligatoires pour le
  # template.
  my @mandatory_radios     = ();
  my @mandatory_checkboxes = ();
  
  for my $fields (values %{$res->{'byhtmlname'}}) {
    if (scalar(@$fields) > 1) {
      # Propagation de certains attributs
      my $f;
      my ($mandatory, $mandatory_error, $keep, $regexp, $validate) = 0;
      
      for $f (@$fields) {
        if ($f->{'mandatory'}) {
          $mandatory = 1;
          $mandatory_error = $f->{'mandatory_error'};
        }
        $keep     = 1 if $f->{'keep'};
        $regexp   = $f->{'regexp'} if ($f->{'regexp'});
        $validate = $f->{'validate'} if ($f->{'validate'});
      }
      
      for $f (@$fields) {
        next if $f->{'disabled'};
        
        if ($mandatory) {
          $f->{'mandatory'}       = 1;
          $f->{'mandatory_error'} = $mandatory_error;
        }
        $f->{'keep'}     = 1 if ($keep);
        $f->{'regexp'}   = $regexp if ($regexp);
        $f->{'validate'} = $validate if ($validate);
      }
    }
    
    # Maintenant, les radios et checboxes obligatoires pour le template
    if ($fields->[0]->{'mandatory'} &&
        ($fields->[0]->{'type_radio'} || $fields->[0]->{'type_checkbox'})) {
      # On fait une copie, sinon, byhtmlname et mandatory_* partagent le
      # mme tableau, c'est chiant  maintenir.
      # XXX Avec ou sans les disabled ?
      #my @fields_copy = grep { !($_->{'disabled'}) } @$fields;
      my @fields_copy = @$fields;
      
      my %struct = (
        'input_name'      => $fields->[0]->{'input_name'},
        'input_id'        => $fields->[0]->{'input_id'},
        'mandatory_error' => $fields->[0]->{'mandatory_error'},
        'inputs'          => \@fields_copy,
      );
      
      if ($fields->[0]->{'type_radio'}) {
        push @mandatory_radios, \%struct;
      } else {
        push @mandatory_checkboxes, \%struct;
      }
    }
  }
  
  $res->{'mandatory_radios'}     = \@mandatory_radios;
  $res->{'mandatory_checkboxes'} = \@mandatory_checkboxes;
  
  return $res;
}

#
# Ajoute une option  un champ `select'
#
sub form_add_selval ($$;$) {
  my ($select, $selval, $where) = @_;
  
  unless (defined $select && $select->{'type_select'} &&
          defined $selval && $selval->{'selval_name'}) {
    error "Les paramtres ne sont pas ceux attendus.";
    return;
  }
  
  # Le positionnement est au choix :
  #  - BEGIN
  #  - END (par dfaut)
  #  - name[<>]$name
  unless ($where && $where =~ /^(BEGIN|END|(name|id)[<>].+)$/o) {
    $where = 'END';
  }
  
  # Vrifie que la liste des champs existe bien (pour viter un
  # warning ou pire sur le drfrencement plus bas).
  unless (defined $select->{'input_values'} &&
          UNIVERSAL::isa($select->{'input_values'}, 'ARRAY')) {
    $select->{'input_values'} = [];
  }
  
  # Ajoute l'option parmi ses potes
  my @values = @{$select->{'input_values'}};
  
  if ($where eq 'BEGIN') {
    unshift @values, $selval;
  } elsif ($where eq 'END') {
    push @values, $selval;
  } else {
    # Premire tape, recherche du voisin
    my ($attrib, $side, $neighbour) = $where =~ /(\w+)([<>])(\w+)/o;
    if ($attrib eq 'id') {
      $attrib = 'option_id';
    } else {
      $attrib = 'selval_name';
    }
    
    my $i;
    my $found = 0;
    for ($i = 0; $i < scalar @values; ++$i) {
      if ($values[$i]->{$attrib} eq $neighbour) {
        $found = 1;
        last;
      }
    }
    
    # $i correspond au voisin qui nous veut du bien, installons ce champ
    # juste  ct (ou  la fin si le voisin n'a pas t trouv).
    if ($side eq '<' && $found) {
      # Avant
      if ($i == 0) {
        unshift @values, $selval;
      } else {
        splice @values, $i, 0, $selval;
      }
    } else {
      # Aprs
      if ($i == $#values) {
        push @values, $selval;
      } else {
        splice @values, $i + 1, 0, $selval;
      }
    }
  }
  
  $select->{'input_values'} = \@values;
}

#
# Ajoute un champ  un bloc, en fonction du positionnemant donn.
#
sub form_add_field ($$;$) {
  my ($block, $field, $where) = @_;
  
  unless (defined $block && UNIVERSAL::isa($block, 'HASH') &&
          defined $field && UNIVERSAL::isa($field, 'HASH') &&
          $block->{'block_name'} && $field->{'input_name'}) {
    error "Les paramtres ne sont pas ceux attendus.";
    return;
  }
  
  # Le positionnement est au choix :
  #  - BEGIN
  #  - END (par dfaut)
  #  - name[<>]$name
  #  - html[<>]$htmlname
  #  - id[<>]$id
  unless ($where && $where =~ /^(BEGIN|END|(name|html|id)[<>].+)$/o) {
    $where = 'END';
  }
  
  # Vrifie que la liste des champs existe bien (pour viter un
  # warning ou pire sur le drfrencement plus bas).
  unless (defined $block->{'block_elems'} &&
          UNIVERSAL::isa($block->{'block_elems'}, 'ARRAY')) {
    $block->{'block_elems'} = [];
  }
  
  # Ajoute le champ parmi ses potes
  my @fields = @{$block->{'block_elems'}};
  
  if ($where eq 'BEGIN') {
    # Ajout au dbut de la liste des champs
    unshift @fields, $field;
  } elsif ($where eq 'END' || scalar @fields == 0) {
    # Ajout en fin de liste des champs
    push @fields, $field;
  } else {
    # Premire tape, recherche du voisin
    my ($attrib, $side, $neighbour) = $where =~ /(\w+)([<>])(\w+)/o;
    if ($attrib eq 'id') {
      $attrib = 'input_id';
    } elsif ($attrib eq 'html') {
      $attrib = 'input_name';
    } else {
      $attrib = 'field_name';
    }
    
    my $i;
    my $found = 0;
    for ($i = 0; $i < scalar @fields; ++$i) {
      if ($fields[$i]->{$attrib} eq $neighbour) {
        $found = 1;
        last;
      }
    }
    
    # Deuxime tape, ajout. $i correspond au voisin qui nous veut du
    # bien, installons ce champ juste  ct (ou  la fin si le voisin
    # n'a pas t trouv).
    if ($side eq '<' && $found) {
      # Avant
      if ($i == 0) {
        unshift @fields, $field;
      } else {
        splice @fields, $i, 0, $field;
      }
    } else {
      # Aprs
      if ($i >= $#fields) {
        push @fields, $field;
      } else {
        splice @fields, $i + 1, 0, $field;
      }
    }
  }
  
  $block->{'block_elems'} = \@fields;
  $field->{'parent_block'} = $block;
  
  # Met  jour les tables de correspondances
  __add_field_ref($field);
  
  # Il ne reste plus qu' revoir les tabindex
  my $form = $field->{'parent_block'}->{'parent_part'}->{'parent_form'};
  __recompute_tabindex($form);
}

#
# Ajoute un bloc  une partie, en fonction du positionnement donn.
#
sub form_add_block ($$;$) {
  my ($part, $block, $where) = @_;
  unless (defined $part  && UNIVERSAL::isa($part, 'HASH') &&
          defined $block && UNIVERSAL::isa($block, 'HASH') &&
          $part->{'part_name'} && $block->{'block_name'}) {
    error "Les paramtres ne sont pas ceux attendus.";
    return;
  }
  
  # Le positionnement est au choix :
  #  - BEGIN
  #  - END (par dfaut)
  #  - name[<>]$name
  #  - id[<>]$id
  unless ($where && $where =~ /^(BEGIN|END|(name|id)[<>].+)$/o) {
    $where = 'END';
  }
  
  # Vrifie que la liste des champs existe bien (pour viter un
  # warning ou pire sur le drfrencement plus bas).
  unless (defined $part->{'blocks'} &&
          UNIVERSAL::isa($part->{'blocks'}, 'ARRAY')) {
    $part->{'blocks'} = [];
  }
  
  # Ajoute le champ parmi ses potes
  my @blocks = @{$part->{'blocks'}};
  
  if ($where eq 'BEGIN') {
    unshift @blocks, $block;
  } elsif ($where eq 'END') {
    push @blocks, $block;
  } else {
    # Premire tape, recherche du voisin
    my ($attrib, $side, $neighbour) = $where =~ /(\w+)([<>])(\w+)/o;
    if ($attrib eq 'id') {
      $attrib = 'block_id';
    } else {
      $attrib = 'block_name';
    }
    
    my $i;
    my $found = 0;
    for ($i = 0; $i < scalar @blocks; ++$i) {
      if ($blocks[$i]->{$attrib} eq $neighbour) {
        $found = 1;
        last;
      }
    }
    
    # Deuxime tape, ajout. $i correspond au voisin qui nous veut du
    # bien, installons ce champ juste  ct (ou  la fin si le voisin
    # n'a pas t trouv).
    if ($side eq '<' && $found) {
      # Avant
      if ($i == 0) {
        unshift @blocks, $block;
      } else {
        splice @blocks, $i, 0, $block;
      }
    } else {
      # Aprs
      if ($i >= $#blocks) {
        push @blocks, $block;
      } else {
        splice @blocks, $i + 1, 0, $block;
      }
    }
  }
  
  $part->{'blocks'} = \@blocks;
  $block->{'parent_part'} = $part;
  
  # Met  jour les tables de correspondances
  for my $field (@{$block->{'block_elems'}}) {
    __add_field_ref($field, 2);
  }
  
  __add_block_ref($block);
  
  # Il ne reste plus qu' revoir les tabindex
  my $form = $block->{'parent_part'}->{'parent_form'};
  __recompute_tabindex($form);
}

#
# Ajoute une partie  un formulaire.
#
sub form_add_part ($$) {
  my ($form, $part) = @_;
  unless (defined $form && UNIVERSAL::isa($form, 'HASH') &&
          defined $part && UNIVERSAL::isa($part, 'HASH') &&
          $form->{'form_name'} && $part->{'part_name'}) {
    error "Les paramtres ne sont pas ceux attendus.";
    return;
  }
  
  # Vrifie que la liste des champs existe bien (pour viter un
  # warning ou pire sur le drfrencement plus bas).
  unless (defined $form->{'parts'} &&
          UNIVERSAL::isa($form->{'parts'}, 'ARRAY')) {
    $form->{'parts'} = [];
  }
  
  # Ajoute le champ parmi ses potes
  push @{$form->{'parts'}}, $part;
  $part->{'parent_form'} = $form;
  
  # Met  jour les tables de correspondances
  for my $block (@{$part->{'blocks'}}) {
    for my $field (@{$block->{'block_elems'}}) {
      __add_field_ref($field, 1);
    }
    __add_block_ref($block, 1);
  }
  
  __add_part_ref($part);
  
  # Il ne reste plus qu' revoir les tabindex
  __recompute_tabindex($form);
}

#
# Supprime une option (selval) d'un select.
#
sub form_del_selval ($$;$) {
  my ($select, $selval_name, $bywhat) = @_;
  
  unless (defined $select && $select->{'type_select'} && $selval_name) {
    error "Les paramtres ne sont pas ceux attendus.";
    return;
  }
  
  $bywhat = 'name' unless ($bywhat && $bywhat =~ /^(name|id)$/o);
  my $attrib = ($bywhat eq 'id') ? 'option_id' : 'selval_name';
  
  unless (defined $select->{'input_values'} &&
          UNIVERSAL::isa($select->{'input_values'}, 'ARRAY')) {
    info "Ce select n'a aucune valeur";
    return;
  }
  
  my @values = @{$select->{'input_values'}};
  
  my $i;
  my $found = 0;
  for ($i = 0; $i < scalar @values; ++$i) {
    if ($values[$i]->{$attrib} eq $selval_name) {
      $found = 1;
      last;
    }
  }
  
  if ($found) {
    splice @values, $i, 1;
    $select->{'input_values'} = \@values;
  }
}

#
# Supprime un champ ou un groupe de champs d'un formulaire.
#
sub form_del_field ($$;$) {
  my ($form, $field_name, $bywhat) = @_;
  
  unless (defined $form && UNIVERSAL::isa($form, 'HASH') && $field_name) {
    error "Les paramtres ne sont pas ceux attendus.";
    return;
  }
  
  $bywhat = 'name' unless ($bywhat && $bywhat =~ /^(name|html|id)$/o);
  
  if ($bywhat eq 'html') {
    my @fields = @{$form->{'byhtmlname'}->{$field_name}};
    
    for my $field (@fields) {
      __delete_field_ref($field);
    }
  } else {
    my $field;
    
    if ($bywhat eq 'id') {
      $field = $form->{'byid'}->{$field_name};
    } else {
      $field = $form->{'byname'}->{$field_name};
    }
    
    __delete_field_ref($field);
  }
  
  # Il ne reste plus qu' revoir les tabindex
  __recompute_tabindex($form);
}

#
# Supprime un bloc (et tous ses champs) d'une partie.
#
sub form_del_block ($$;$) {
  my ($part, $block_name, $bywhat) = @_;
  
  unless (defined $part && $part->{'part_name'} && $block_name) {
    info "Les paramtres ne sont pas ceux attendus.";
    return;
  }
  
  $bywhat = 'name' unless ($bywhat && $bywhat =~ /^(name|id)$/o);
  my $attrib = ($bywhat eq 'id') ? 'block_id' : 'block_name';
  
  unless (defined $part->{'blocks'} &&
          UNIVERSAL::isa($part->{'blocks'}, 'ARRAY')) {
    info "Partie sans bloc. Termin.";
    return;
  }
  
  my @blocks = @{$part->{'blocks'}};
  
  my $i;
  my $found = 0;
  for ($i = 0; $i < scalar @blocks; ++$i) {
    if ($blocks[$i]->{$attrib} eq $block_name) {
      $found = 1;
      last;
    }
  }
  
  if ($found) {
    # On supprime le bloc de la liste
    my $block = splice @blocks, $i, 1;
    
    # S'il a des champs, on les supprime aussi
    if ($block->{'block_elems'} &&
        UNIVERSAL::isa($block->{'block_elems'}, 'ARRAY')) {
      # Tous les champs de ce bloc doivent tre supprims un par un
      my @fields = @{$block->{'block_elems'}};
      for my $f (@fields) {
        form_del_field($part->{'parent_form'}, $f->{'field_name'});
      }
    }
    $part->{'blocks'} = \@blocks;
    
    # Enfin, on met  jour les tables de corespondanes
    __delete_block_ref($block);
  }
}

#
# Supprime une partie d'un formulaire.
#
sub form_del_part ($$) {
  my ($form, $part_name) = @_;
  
  unless (defined $form && $form->{'form_name'} && $part_name) {
    info "Les paramtres ne sont pas ceux attendus.";
    return;
  }
  
  unless (defined $form->{'parts'} &&
          UNIVERSAL::isa($form->{'parts'}, 'ARRAY')) {
    info "Partie sans bloc. Termin.";
    return;
  }
  
  my @parts= @{$form->{'parts'}};
  
  my $i;
  my $found = 0;
  for ($i = 0; $i < scalar @parts; ++$i) {
    if ($parts[$i]->{'part_name'} eq $part_name) {
      $found = 1;
      last;
    }
  }
  
  if ($found) {
    # On supprime la partie de la liste
    my $part = splice @parts, $i, 1;
    
    # Si elle a des blocs, on les supprime
    if ($part->{'blocks'} &&
        UNIVERSAL::isa($part->{'blocks'}, 'ARRAY')) {
      # Tous les blocs de cette partie doivent tre supprims un par un
      for my $b (@{$part->{'blocks'}}) {
        form_del_block($part, $b->{'block_name'});
      }
    }
    $form->{'parts'} = \@parts;
    
    # Enfin, on met  jour les tables de corespondanes
    __delete_part_ref($part);
  }
}

#
# Pour un champ donn, change sa valeur (type text-like ou button-like)
# ou la(les) valeur(s) slectionne(s) (type checkbox, radio ou select).
#
sub form_set_field_value ($$) {
  my ($field, $value) = @_;
  
  if (defined $field->{'type_select'} && $field->{'type_select'}) {
    form_set_select_value($field, $value);
  } elsif ($field->{'type_text'} || $field->{'type_password'}) {
    form_set_text_value($field, $value);
  } elsif (defined $field->{'type_textarea'} && $field->{'type_textarea'}) {
    form_set_textarea_value($field, $value);
  } elsif ($field->{'type_button'} ||
           $field->{'type_submit'} ||
           $field->{'type_reset'}) {
    form_set_button_value($field, $value);
  } elsif (defined $field->{'type_checkbox'} && $field->{'type_checkbox'}) {
    form_set_checkbox_value($field, $value);
  } elsif (defined $field->{'type_radio'} && $field->{'type_radio'}) {
    form_set_radio_value($field, $value);
  } elsif (defined $field->{'type_html'} && $field->{'type_html'}) {
    form_set_html_value($field, $value);
  } elsif (defined $field->{'type_hidden'} && $field->{'type_hidden'}) {
    form_set_hidden_value($field, $value);
  } else {
    warning "Je ne sais pas positionner la valeur d'un champs ".
      $field->{'input_type'}." (".
      ($field->{'input_name'} ? $field->{'input_name'} : 'inconnu').")";
  }
}

# --------------------------------------------------------------------
# Fonctions utilitaires
# --------------------------------------------------------------------

sub str_http_decode ($) {
  my $url = shift;
  return unless defined $url;
  
  # Change + signs to spaces 
  $url =~ tr/+/ /;
  
  # Change hex escapes to the proper characters 
  $url =~ s/%([a-fA-F0-9]{2})/pack "H2", $1/eg;
  
  return $url; 
}

sub str_http_encode ($) {
  my $url = shift;
  return unless defined $url;
  
  # Change unsafe characters (except for space) to encoded value
  $url =~ s/([^ a-zA-Z0-9._\-!~*'])/sprintf '%%%02X', ord($1)/eg;
  
  # Change spaces to +
  $url =~ tr/ /+/;
  
  return $url; 
}

sub str_js_escape ($) {
  my ($text) = @_;
  
  # Echappe certains caractres our Javascript (le template met les chanes
  # entre double-quotes)
  $text =~ s/"/'/og;
  
  return $text;
}

# --------------------------------------------------------------------
# Fonctions internes de cration de champ, bloc ou partie
# --------------------------------------------------------------------

sub form_read_select ($$) {
  my ($config, $field) = @_;
  my $res;
  
  # On pose le type et la liste vide des valeurs
  $res = {
    'type_select'  => 1,
    'input_values' => []
  };
  
  # Gestion des select multiple
  if (defined $field->{'multiple'} && $field->{'multiple'} eq 'true') {
    $res->{'multiple'} = 1;
  }
  
  # Gestion des select avec attribut onchange
  if ($field->{'onchange'}) {
    $res->{'onchange'} = $field->{'onchange'};
  }
  
  # Pour chaque valeur, on l'ajoute dans la liste
  for my $sv (@{$field->{'@values'}}) {
    debug "Le select `".$field->{'name'}."' contient la valeur `$sv'"
      if ($DEBUG);
    
    my $val = form_new_selval($config, $sv);
    unless (defined $val) {
      error "La selval `$sv' du select `".$field->{'field_name'}.
        "' est inconnue";
      next;
    }
    
    # Valeur par dfaut ?
    if (defined $field->{'selected'} && !$val->{'disabled'}) {
      if ($val->{'optgroup'}) {
        for my $opt (@{$val->{'options'}}) {
          if ($opt->{'selval_name'} eq $field->{'selected'} &&
            !$opt->{'disabled'}) {
            $opt->{'selected'} = 1;
            last;
          }
        }
      } elsif ($sv eq $field->{'selected'}) {
        $val->{'selected'} = 1;
      }
    }
    
    # Ajouter a la liste
    push @{$res->{'input_values'}}, $val;
  }
  
  return $res;
}

sub form_read_text ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_text' => 1 };
  
  # La longueur du champ
  if (defined $field->{'size'}) {
    $res->{'input_size'} = $field->{'size'};
  }
  
  # La longueur max du champ
  if (defined $field->{'maxsize'}) {
    $res->{'input_maxsize'} = $field->{'maxsize'};
  }
  
  # La valeur par defaut
  if (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  }
  
  # Est-il en lecture seule ?
  if (defined $field->{'readonly'} && $field->{'readonly'} eq 'true') {
    $res->{'readonly'} = 1;
    
    if (defined ($res->{'regexp'})) {
      debug "Le champ `".$field->{'name'}."' est readonly, mais a une regexp"
        if ($DEBUG);
    }
    if (!$res->{'input_value'} &&
      defined $field->{'mandatory'} && $field->{'mandatory'} eq 'true') {
      debug "Le champ `".$field->{'name'}.
        "' est readonly, obligatoire et sans valeur" if ($DEBUG);
    }
=begin NON !
    # Pour viter tout problme avec un champ en lecture seule, on supprime
    # une ventuelle vrification de syntaxe.
    $res->{'regexp'} = undef;
    
    # On supprime aussi le mandatory si sa valeur est vide
    unless ($res->{'input_value'}) {
      $res->{'mandatory'} = 0;
    }
=cut
  }
  
  return $res;
}

sub form_read_password ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_password' => 1 };
  
  # La longueur du champ
  if (defined $field->{'size'}) {
    $res->{'input_size'} = $field->{'size'};
  }
  
  # La valeur par defaut
  if (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  }
  
  return $res;
}

sub form_read_textarea ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_textarea' => 1 };
  
  # Contenu du textarea
  if (defined $field->{'file'}) {
    $res->{'content'} = remplace($field->{'file'}, $field, $field);
  } elsif (defined $field->{'content'}) {
    $res->{'content'} = parse($field->{'content'}, $field, $field);
  }
  
  # Est-il en lecture seule ?
  if (defined $field->{'readonly'} && $field->{'readonly'} eq 'true') {
    $res->{'readonly'} = 1;

    if (defined ($res->{'regexp'})) {
      debug "Le champ `".$field->{'name'}."' est readonly, mais a une regexp"
        if ($DEBUG);
    }
    if (!$res->{'input_value'} &&
      defined $field->{'mandatory'} && $field->{'mandatory'} eq 'true') {
      debug "Le champ `".$field->{'name'}.
        "' est readonly, obligatoire et sans valeur" if ($DEBUG);
    }
=begin NON !
    # Pour viter tout problme avec un champ en lecture seule, on supprime
    # une ventuelle vrification de syntaxe.
    $res->{'regexp'} = undef;
    
    # On supprime aussi le mandatory si sa valeur est vide
    unless ($res->{'input_value'}) {
      $res->{'mandatory'} = 0;
    }
=cut
  }
  
  # Taille
  if (defined $field->{'cols'} && $field->{'cols'} > 0) {
    $res->{'cols'} = $field->{'cols'};
  }
  if (defined $field->{'rows'} && $field->{'rows'} > 0) {
    $res->{'rows'} = $field->{'rows'};
  }
  
  return $res;
}

sub form_read_hidden ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_hidden' => 1 };
  
  # Ref vers le formulaire, utilis pour form_keep_val()
  $res->{'form'} = $reading_form;
  
  if (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  }
  
  return $res;
}

sub form_read_image ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_image' => 1 };
  
  # L'URL de l'image
  if (defined $field->{'src'}) {
    $res->{'input_src'} = $field->{'src'};
  } else {
    error "Pas d'image source pour le champ ".$field->{'name'};
  }
  
  # La valeur
  if (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  } else {
    $res->{'input_value'} = $field->{'name'};
  }
  
  # Le texte alternatif
  if (defined $field->{'alt'}) {
    $res->{'input_alt'} = $field->{'alt'};
  } else {
    $res->{'input_alt'} = $res->{'input_value'};
  }
  
  # L'action Javascript pour onclick
  if (defined $field->{'onclick'}) {
    $res->{'submit_onclick'} = $field->{'onclick'};
    $res->{'submit_onclick'} = str_js_escape($res->{'submit_onclick'});
  }
  
  # On dsactive keep, car le champ ne le supporte pas
  $res->{'keep'} = 0;
  
  return $res;
}

sub form_read_resetimage ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_resetimage' => 1 };
  
  # L'URL de l'image
  if (defined $field->{'src'}) {
    $res->{'input_src'} = $field->{'src'};
  } else {
    error "Pas d'image source pour le champ ".$field->{'name'};
  }
  
  # Le texte alternatif
  if (defined $field->{'alt'}) {
    $res->{'input_alt'} = $field->{'alt'};
  } else {
    info "Pas de alt pour le champ ".$field->{'name'};
  }
  
  # On dsactive keep, car le champ ne le supporte pas
  $res->{'keep'} = 0;
  
  return $res;
}

sub form_read_checkbox ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_checkbox' => 1 };
  
  # Ref vers le formulaire, utilis pour form_keep_val()
  $res->{'form'} = $reading_form;
  
  # La valeur
  if (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  } else {
    debug "Le checkbox `".$field->{'name'}.
    "' n'a pas de value, on la force  1" if ($DEBUG);
    $res->{'input_value'} = 1;
  }
  
  # Est-il coch ?
  # Attention, on ne vrifie pas s'il est disabled !
  if (defined $field->{'checked'} && $field->{'checked'} eq 'true') {
    $res->{'checked'} = 1;
  } elsif (defined $field->{'disabled'} && $field->{'disabled'} eq 'true' &&
      defined $field->{'mandatory'} && $field->{'mandatory'} eq 'true') {
    debug "Le checkbox `".$field->{'name'}.
    "' n'est pas checked mais disabled et obligatoire" if ($DEBUG);
  }
  
  # L'id
  unless ($field->{'id'}) {
    $res->{'input_id'} = $field->{'name'}.$res->{'input_value'};
  }
  
  # L'action Javascript pour onclick
  if (defined $field->{'onclick'}) {
    $res->{'checkbox_onclick'} = $field->{'onclick'};
    $res->{'checkbox_onclick'} = str_js_escape($res->{'checkbox_onclick'});
  }
  
  return $res;
}

sub form_read_radio ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_radio' => 1 };
  
  # Ref vers le formulaire, utilis pour form_keep_val()
  $res->{'form'} = $reading_form;
  
  # La valeur (obligatoire)
  if (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  } else {
    error "Le radio `".$field->{'name'}."' n'a pas de value";
    return;
  }
  
  # Est-il coch ?
  if (defined $field->{'checked'} && $field->{'checked'} eq 'true') {
    $res->{'checked'} = 1;
  } elsif (defined $field->{'disabled'} && $field->{'disabled'} eq 'true' &&
      defined $field->{'mandatory'} && $field->{'mandatory'} eq 'true') {
    debug "Le radio `".$field->{'name'}.
    "' n'est pas checked mais disabled et obligatoire" if ($DEBUG);
  }
  
  # L'id
  unless ($field->{'id'}) {
    $res->{'input_id'} = $field->{'name'}.$res->{'input_value'};
  }
  
  # L'action Javascript pour onclick
  if (defined $field->{'onclick'}) {
    $res->{'radio_onclick'} = $field->{'onclick'};
    $res->{'radio_onclick'} = str_js_escape($res->{'radio_onclick'});
  }
  
  return $res;
}

sub form_read_button ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_button' => 1 };
  
  # La valeur (ce qui est crit sur le bouton)
  if (defined $field->{'label'}) {
    $res->{'input_value'} = $field->{'label'};
  } elsif (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  } else {
    $res->{'input_value'} = $field->{'name'};
  }
  
  # L'action Javascript pour onclick
  if (defined $field->{'onclick'}) {
    $res->{'submit_onclick'} = $field->{'onclick'};
    $res->{'submit_onclick'} = str_js_escape($res->{'submit_onclick'});
  }
  
  return $res; 
}

sub form_read_file ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_file' => 1 };
  
  # La longueur du champ
  if (defined $field->{'size'}) {
    $res->{'input_size'} = $field->{'size'};
  }
  
  # Les mimetypes accepts
  if (defined $field->{'accept'}) {
    $res->{'accept'} = $field->{'accept'};
  }
  
  # On dsactive keep, car le champ ne le supporte pas
  $res->{'keep'} = 0;
  
  return $res;
}

sub form_read_submit ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_submit' => 1 };
  
  # La valeur (ce qui est crit sur le bouton)
  if (defined $field->{'label'}) {
    $res->{'input_value'} = $field->{'label'};
  } elsif (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  } else {
    $res->{'input_value'} = $field->{'name'};
  }
  
  # L'action Javascript pour onclick
  if (defined $field->{'onclick'}) {
    $res->{'submit_onclick'} = $field->{'onclick'};
    $res->{'submit_onclick'} = str_js_escape($res->{'submit_onclick'});
  }
  
  return $res; 
}

sub form_read_reset ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_reset' => 1 };
  
  # La valeur (ce qui est crit sur le bouton)
  if (defined $field->{'label'}) {
    $res->{'input_value'} = $field->{'label'};
  } elsif (defined $field->{'value'}) {
    $res->{'input_value'} = $field->{'value'};
  } else {
    $res->{'input_value'} = $field->{'name'};
  }
  
  # L'action Javascript pour onclick
  if (defined $field->{'onclick'}) {
    $res->{'submit_onclick'} = $field->{'onclick'};
    $res->{'submit_onclick'} = str_js_escape($res->{'submit_onclick'});
  }
  
  return $res; 
}

sub form_read_emptycell ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_emptycell' => 1 };
  
  $res->{'width'}  = $field->{'width'};
  $res->{'height'} = $field->{'height'};
  
  return $res;
}

sub form_read_html ($$) {
  my ($config, $field) = @_;
  my $res = { 'type_html' => 1 };
  
  if (defined $field->{'file'}) {
    $res->{'content'} = remplace($field->{'file'}, $field, $field);
  } elsif (defined $field->{'content'}) {
    $res->{'content'} = parse($field->{'content'}, $field, $field);
  }
  
  return $res;
}

# --------------------------------------------------------------------
# Fonctions internes de modification de valeur(s) de champ
# --------------------------------------------------------------------

sub form_set_select_value ($$) {
  my ($field, $value) = @_;
  my $padded_value =
    UNIVERSAL::isa($value, 'ARRAY') ? join("\0", @$value) : $value;
  $padded_value = "\0".$padded_value."\0";
  
  for my $selv (@{$field->{'input_values'}}) {
    my $opt_value = $selv->{'option_value'};
    # \Q desactive l'interpretation des caracteres speciaux des regexp jusqu'a \E.
    # Permet d'avoir '+=' pour $opt_value
    if ($padded_value =~ /\0(\Q$opt_value\E)\0/) {
      $selv->{'selected'} = 1;
    } else {
      delete $selv->{'selected'} if (defined($selv->{'selected'}));
    }
  }
}

sub form_set_checkbox_value ($$) {
  my ($field, $value) = @_;
  
  my %values = ();
  if (UNIVERSAL::isa($value, 'ARRAY')) {
    %values = map { $_ => 1 } @$value;
  } else{
    # CGI.pm stocke les valeurs multiples sous forme de chaine dont les
    # valeurs sont spares par des \0 (caractre NUL du C).
    my @vals = split /\0/, $value;
    %values = map { $_ => 1 } @vals;
  }
  
  my $name = $field->{'input_name'};
  for my $n (@{$field->{'form'}->{'byhtmlname'}->{$name}}) {
    my $opt_value = $n->{'input_value'};
    if ($values{$opt_value}) {
      $n->{'checked'} = 1;
    } else {
      delete $n->{'checked'} if (defined $n->{'checked'});
    }
  }
}

sub form_set_radio_value ($$) {
  my ($field, $value) = @_;
  
  my $name = $field->{'input_name'};
  debug "Nouvelle slection pour le groupe de radios `$name' : ".
  ($value ? $value : 'aucun');
  for my $n (@{$field->{'form'}->{'byhtmlname'}->{$name}}) {
    if ($value && $n->{'input_value'} eq $value) {
      $n->{'checked'} = 1;
    } else {
      delete $n->{'checked'} if (defined $n->{'checked'});
    }
  }
}

sub form_set_text_value ($$) {
  my ($field, $value) = @_;
  # viter de laisser la valeur sortir de la balise
  $value =~ s/"//g;
  $field->{'input_value'} = $value;
}

sub form_set_textarea_value ($$) {
  my ($field, $value) = @_;
  # viter de laisser la valeur sortir de la balise
  $value =~ s/"//g;
  $field->{'content'} = parse($value, $field, $field);
}

sub form_set_html_value ($$) {
  my ($field, $value) = @_;
  $field->{'content'} = parse($value, $field, $field);
}

sub form_set_button_value ($$) {
  my ($field, $value) = @_;
  # viter de laisser la valeur sortir de la balise
  $value =~ s/"//g;
  $field->{'input_value'} = $value;
}

sub form_set_hidden_value ($$) {
  my ($field, $value) = @_;
  
  my @values = ();
  if (UNIVERSAL::isa($value, 'ARRAY')) {
    @values = @$value;
  } else{
    # CGI.pm stocke les valeurs multiples sous forme de chaine dont les
    # valeurs sont spares par des \0 (caractre NUL du C).
    @values = split /\0/, $value;
  }
  
  my $name = $field->{'input_name'};
  for my $n (@{$field->{'form'}->{'byhtmlname'}->{$name}}) {
    my $v = shift @values;
    # viter de laisser la valeur sortir de la balise
    $v =~ s/"//g;
    $n->{'input_value'} = $v;
  }
}

# --------------------------------------------------------------------
# Fonctions internes diverses
# --------------------------------------------------------------------

sub __add_field_ref ($;$) {
  my ($field, $start) = @_;
  return unless (defined $field && $field->{'field_name'});
  
  # $start est le niveau de dpart dans l'arbre :
  #   1 = seulement le formulaire
  #   2 = le formulaire et la partie
  #   3 = le formulaire, la partie et le bloc
  $start = 3 unless ($start && $start >= 1);
  
  # Met  jour les tables de correspondances
  #  - au niveau du bloc
  my $block = $field->{'parent_block'};
  __add_field_to($field, $block) if ($start >= 3);
  
  #  - au niveau de la partie
  my $part = $block->{'parent_part'};
  __add_field_to($field, $part) if ($start >= 2);
  
  #  - au niveau du formulaire
  my $form = $part->{'parent_form'};
  __add_field_to($field, $form) if ($start >= 1);
  
  #  - dans les tables de mandatory
  if ($field->{'type_radio'} || $field->{'type_checkbox'}) {
    my $i;
    my $found = 0;
    
    my @mandatories = $field->{'type_radio'} ?
      @{$form->{'mandatory_radios'}} : @{$form->{'mandatory_checkboxes'}};
    
    $found = 0;
    for ($i = 0; $i < scalar @mandatories; ++$i) {
      if ($field->{'input_name'} eq $mandatories[$i]->{'input_name'}) {
        $found = 1;
        last;
      }
    }
    
    if ($found) {
      push @{$mandatories[$i]->{'inputs'}}, $field;
    } elsif ($field->{'mandatory'}) {
      my @fields_copy = @{$form->{'byhtmlname'}->{$field->{'input_name'}}};
      push @mandatories, {
        'input_id'        => $field->{'input_id'},
        'input_name'      => $field->{'input_name'},
        'mandatory_error' => $field->{'mandatory_error'},
        'inputs'          => \@fields_copy,
      };
    }
    
    if ($field->{'type_radio'}) {
      $form->{'mandatory_radios'} = \@mandatories;
    } else {
      $form->{'mandatory_checkboxes'} = \@mandatories;
    }
  }
}

sub __add_field_to ($$) {
  my ($field, $container) = @_;
  return unless (defined $field && $field->{'field_name'});
  
  if (exists $container->{'byid'}->{$field->{'input_id'}}) {
    warning 'Le champ `'.$field->{'field_name'}.
    '\' a le mme id que le champ `'.
    $container->{'byid'}->{$field->{'input_id'}}->{'field_name'}.'\'';
  }
  $container->{'byid'}->{$field->{'input_id'}} = $field;
  
  if (exists $container->{'byname'}->{$field->{'field_name'}}) {
    warning 'Le champ `'.$field->{'field_name'}.
    '\' a le mme nom qu\'un autre champ';
  }
  $container->{'byname'}->{$field->{'field_name'}} = $field;
  
  if (exists $container->{'byhtmlname'}->{$field->{'input_name'}} &&
      !($field->{'type_checkbox'} ||
        $field->{'type_radio'} ||
        $field->{'type_hidden'})) {
    warning 'Le champ '.$field->{'field_name'}.
    ' a le mme nom HTML qu\'un autre champ';
  }
  push @{$container->{'byhtmlname'}->{$field->{'input_name'}}}, $field;
}

sub __add_block_ref ($;$) {
  my ($block, $start) = @_;
  
  $start = 2 unless ($start && $start >= 1);
  
  my $part = $block->{'parent_part'};
  $part->{'blocksbyname'}->{$block->{'block_name'}} = $block if ($start >= 2);
  
  my $form = $part->{'parent_form'};
  $form->{'blocksbyname'}->{$block->{'block_name'}} = $block if ($start >= 1);
}

sub __add_part_ref ($) {
  my ($part) = @_;
  
  my $form = $part->{'parent_form'};
  $form->{'partsbyname'}->{$part->{'part_name'}} = $part;
}

sub __delete_field_ref ($) {
  my ($field) = @_;
  return unless (defined $field && $field->{'field_name'});
  
  my $i;
  my $found = 0;
  
  # Supprime le champ des tables de correspondance
  #  - au niveau du bloc
  my $block = $field->{'parent_block'};
  __delete_field_from($field, $block);
  
  my $elems = $block->{'block_elems'};
  for ($i = 0; $i < scalar @$elems; ++$i) {
    if ($elems->[$i]->{'field_name'} eq $field->{'field_name'}) {
      $found = 1;
      last;
    }
  }
  splice @$elems, $i, 1 if $found;
  
  #  - au niveau de la partie
  my $part = $block->{'parent_part'};
  __delete_field_from($field, $part);
  
  #  - au niveau du formulaire
  my $form = $part->{'parent_form'};
  __delete_field_from($field, $form);
  
  #  - dans les tables de mandatory
  if ($field->{'type_radio'} || $field->{'type_checkbox'}) {
    my $mandatories = $field->{'type_radio'} ?
      $form->{'mandatory_radios'} : $form->{'mandatory_checkboxes'};
    
    $found = 0;
    for ($i = 0; $i < scalar @$mandatories; ++$i) {
      if ($field->{'input_name'} eq $mandatories->[$i]->{'input_name'}) {
        $found = 1;
        last;
      }
    }
    
    if ($found) {
      my $j;
      my $inputs = $mandatories->[$i]->{'inputs'};
      
      $found = 0;
      for ($j = 0; $j < scalar @$inputs; ++$j) {
        if ($field->{'field_name'} eq $inputs->[$j]->{'field_name'}) {
          $found = 1;
          last;
        }
      }
      splice @$inputs, $j, 1 if $found;
      
      if (scalar @$inputs) {
        if ($field->{'input_id'} eq $mandatories->[$i]->{'input_id'}) {
          $mandatories->[$i]->{'input_id'} = $inputs->[0]->{'input_id'};
        }
      } else {
        splice @$mandatories, $i, 1;
      }
    }
  }
}

sub __delete_field_from ($$) {
  my ($field, $container) = @_;
  return unless (defined $field && $field->{'field_name'});
  
  my $i;
  my $found = 0;
  
  delete $container->{'byid'}->{$field->{'input_id'}};
  delete $container->{'byname'}->{$field->{'field_name'}};
  
  my $byhtmlname = $container->{'byhtmlname'}->{$field->{'input_name'}};
  for ($i = 0; $i < scalar @$byhtmlname; ++$i) {
    if ($field->{'field_name'} eq $byhtmlname->[$i]->{'field_name'}) {
      $found = 1;
      last;
    }
  }
  splice @$byhtmlname, $i, 1 if $found;
  
  unless (scalar @$byhtmlname) {
    delete $container->{'byhtmlname'}->{$field->{'input_name'}};
  }
}

sub __delete_block_ref ($) {
  my ($block) = @_;
  
  my $part = $block->{'parent_part'};
  delete $part->{'blocksbyname'}->{$block->{'block_name'}};
  
  my $form = $part->{'parent_form'};
  delete $form->{'blocksbyname'}->{$block->{'block_name'}};
}

sub __delete_part_ref ($) {
  my ($part) = @_;
  
  my $form = $part->{'parent_form'};
  delete $form->{'partsbyname'}->{$part->{'part_name'}};
}

sub __recompute_tabindex ($) {
  my ($form) = @_;
  $tabindex = 1;
  
  # A partir de l, parcours d'arbre classique : la racine est le
  # formulaire, les feuilles sont les champs.
  my $parts = $form->{'parts'};
  for my $part (@$parts) {
    # On regarde ce que la partie indique sur les tabindex
    if ($part->{'tabindex_start'}) {
      if ($part->{'tabindex_start'} > $tabindex) {
        $tabindex = $part->{'tabindex_start'};
      } else {
        warning "La partie `".$part->{'part_name'}.
          "' indique un tabindex (".$part->{'tabindex_start'}.
          ") trop petit par rapport au prochain disponible ($tabindex)";
      }
    } elsif ($part->{'tabindex_offset'}) {
      $tabindex += $form->{'tabindex_offset'};
    }
    
    # On se lance dans la liste des blocs
    my $blocks = $part->{'blocks'};
    for my $block (@$blocks) {
      my $fields = $block->{'block_elems'};
      for my $field (@$fields) {
        unless ($field->{'disabled'} ||
                $field->{'type_emptycell'} ||
                $field->{'type_hidden'} ||
                $field->{'type_html'}) {
          $field->{'tabindex'} = $tabindex++;
        }
      }
    }
  }
  
  # On met  jour le tabindex disponible dans le form-glob
  $form->{'next_tabindex'} = $tabindex;
}

sub __check_name ($) {
  my ($name) = @_;
  
  my @caller = caller(1);
  my $func   = $caller[3];
  
  if ($name =~ /^\s+/o) {
    warning "Le nom `$name' pass  `$func' contient des espaces au dbut";
  }
  if ($name =~ /\s+$/o) {
    warning "Le nom `$name' pass  `$func' contient des espaces  la fin";
  }
}

1;
