# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
# - Jerome Bourgeois
package FDN::Web::Session;

use strict;
use warnings;

use Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw (
	retrieve_session
	store_session
	close_session
	new_session
	refresh_session
	unlock_session
);

use File::stat;
use Storable qw ( store_fd fd_retrieve );

use FDN::Common::Constantes;
use FDN::Common::Utils;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Common::Modules;

# Pour gnrer le nom du fichier temporaire
my $hostname = `/bin/hostname`;
chomp $hostname;
my $debug = 0;


#############################################################################
#
#   Fonctions internes
#
#############################################################################


# ----------------------------------------------------------------------
# name        : __debug
# description : log un message de debug via le Logger
# parametre   : debug_message = le message a logguer
#               debug = flag indiquant si le message doit etre loggue
#                       ou non
# ----------------------------------------------------------------------
sub __debug ($) {
        my ($debug_message) = @_;
	return unless $debug;
	debug $debug_message;
}               


# ----------------------------------------------------------------------
# name   : __cleaner
# desc   : nettoie un rpertoire de session et ventullement ses N rpertoires parents
#          (suivant $depth)
# param  : $path     = le rpertoire  partir duquel les fichiers de 
#                       session sont nettoys
#          $time_out = dure de validit d'un fichier de session (opt)
#          $prefix   = prfixe des fichiers de session (opt)
#          $depth    = nombre de rpertoires  remonter (opt)
# ----------------------------------------------------------------------
sub __cleaner ($$$;$) {
	my ($path, $time_out, $prefix, $depth) = @_;

	unless ($path) {
		error("Mauvais passage de paramtre");
		return;
	}

	unless ($time_out) {
		error("Pas de date de premption des fichiers de session");
		return;
	}
	
	unless ($prefix) {
		error("Pas de prfixe des fichiers de session");
		return;
	}

	unless ( defined($depth) && $depth >= 0 ) {
		$depth = 0;
	}

	my $ok = 1;
	do {
		__debug "__cleaner dans ".$path;
		if ( opendir( SESSDIR, $path) ) {
			my @fichiers_session = grep /^$prefix/o, readdir SESSDIR;
			__debug "fichiers de session : ".
				join ( ', ', @fichiers_session);
			foreach my $fichier ( @fichiers_session ) {
				my $full_fichier = $path . '/' . $fichier;
				unless ( age_bon ($full_fichier , $time_out) ) {
					__debug " supression du fichier $fichier";
					unlink $full_fichier;
				}
			}
			closedir SESSDIR;
		} else {
			error "opendir $path: $!";
			$ok = 0;
		}
		$path =~ s/\/[^\/]+\/{0,1}$//i; # on remonte d'un repertoire
		$depth--;
	} while ( $depth > 0 );

	return $ok;
}


# ----------------------------------------------------------------------
# name   : __sessalert 
# Envoie un mail d'alerte en cas de problme grave
# ----------------------------------------------------------------------
sub __sessalert ($$$) {
	my ( $do_send, $msg, $to) = @_;
	error $msg;
	if ( $do_send ) {
		conduse('Mail::Sendmail');
		$to = Config_Key('web/session.cfg','global','to') if ( ! defined $to );
		return if ( ! defined $to );
		my $from = Config_Key('web/session.cfg','global','from');
		return if ( ! defined $from );
		my $subject = Config_Key('web/session.cfg','global','subject');
		$subject = 'Session error' if ( ! defined $subject );
		my $msg = Config_Key('web/session.cfg','global','@msg');
		$msg = [ 'Missing /etc/fdn/web/session.cfg' ] if ( ! defined $msg );
		$msg = join "\n", @$msg;
		my $sig = Config_Key('web/session.cfg','global','signature');
		$sig = '' if ( ! defined $sig );
		my %mail = (
			To      => $to ,
			From    => $from,
			Subject => $subject,
			Message => "$msg\n\n".
				   "$sig\n\n"
			);
		sendmail(%mail) or error("__sessalert : pas pu envoyer le mail de warning" );
	}
}


# ----------------------------------------------------------------------
# name   : __init_conf
# desc   : charge la conf a partir du fichier de conf en fonction du 
#          type de session
# params : $type : type de session (opt)
# ----------------------------------------------------------------------
sub __init_conf ($) {
	my ( $type ) = @_;
	
	my $conf = Load_Config('web/session.cfg');
	if ( ! defined $type ) {
		$type = 'default';
		warning("You did not give the session type, using default.");
	}
	if ( ! defined $conf->{$type} ) {
		warning "You requested an undefined session type (".
			$type."), using default instead.";
		$conf->{$type} = {};
	}
	__debug "Type de session : ".$type;
	
	$conf = { %{$conf->{'default'}},
		  %{$conf->{$type}} };
	
	if ( ! defined $conf->{'prefix'} ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no prefix in default)';
		$conf->{'prefix'} = 'sess__';
	}
	
	if ( defined($conf->{'sess_dir'}) && defined($conf->{'base_dir'}) ) {
		$conf->{'path'} = $conf->{'base_dir'}.$conf->{'sess_dir'};
	} else {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no sess_dir and/or base_dir in default)';
		$conf->{'path'} = '/tmp/sessions/'.$conf->{'prefix'};
	}
	
	if ( defined($conf->{'time_out'}) ) {
		if ( $conf->{'time_out'} =~ /^(\d+)([a-z]{1})$/){
			$conf->{'time_out'} = convert_time($1,$2,'s');
		} else {
			error("mauvaise valeur du time_out pour $type, pris 30 minutes ");
			$conf->{'time_out'} = 1800;
		}
	} else {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no time_out in default)';
		$conf->{'time_out'} = 1800;
	}
	
	if ( ! defined($conf->{'hash_depth'}) ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no hash_depth in default)';
		$conf->{'hash_depth'} = 0;
	}
	
	if ( ! defined($conf->{'hash_type'}) ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no hash_type in default)';
		$conf->{'hash_type'} = 'crc32';
	}
	
	if ( ! defined($conf->{'hash_fallback'}) ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no hash_fallback in default)';
		$conf->{'hash_fallback'} = 0;
	}
	if ($conf->{'hash_fallback'} > $conf->{'hash_depth'} ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(hash_fallback>hash_depth => forced to hash_depth)';
		$conf->{'hash_fallback'} = $conf->{'hash_depth'};
	}

	if ( ! defined($conf->{'auto_wash'}) ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no auto_wash in default)';
		$conf->{'auto_wash'} = 1;
	}

	if ( ! defined($conf->{'sess_debug'}) ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no sess_debug in default)';
		$conf->{'sess_debug'} = 0;
	}

	if ( ! defined($conf->{'mail_warn'}) ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no mail_warn in default)';
		$conf->{'mail_warn'} = 0;
	}

	# definition des destinataires du mail d'alerte
	if ( ! defined($conf->{'mail_list'}) ) {
		error 'INVALID CONFIGURATION FILE FOR SESSIONS MAY-DAY '.
		      '(no mail_list in default)';
		$conf->{'mail_mail'} = '';
	}

	if ( ! defined($conf->{'use_lock'}) ) {
		# There is no MAYDAY here, since this option did not use to exist in old version
		warning 'Invalid configuration file for sessions (no use_lock in default)';
		$conf->{'use_lock'} = 0;
	}

	return $conf;
}


# ----------------------------------------------------------------------
# name   : __find_path 
# desc   : trouve dans quel rpertoire est stock un fichier de session
# params : $name = nom de la session
# ----------------------------------------------------------------------
sub __find_path ($$$$$;$) {
	my ( $name, $path, $hash_type, $hash_depth, $hash_fallback, $debug ) = @_;

	unless ($name) {
		error("Mauvais passage de paramtre");
		return;
	}
	
	if ( !defined($path) ) {
		error("path non defini");
		return;
	} 
	
	if ( !defined($hash_depth) ) {
		error("hash_depth non defini");
		return;
	} 
	
	if ( !defined($hash_fallback) ) {
		error("hash_fallback non defini");
		return;
	} 
	
	if ( !defined($hash_type) ) {
		error("hash_type non defini");
		return;
	} 
	
	if ( !defined($debug) ) {
		$debug = 0;
	}
	
	my $ra_hash_dir = safe_dir_hashing($hash_type, $name, $hash_depth, $hash_fallback);
	unless (defined $ra_hash_dir) {
		error("safe_dir_hashing a renvoy undef !");
		return;
	}

	unless (scalar @{$ra_hash_dir}) {
		error("safe_dir_hashing a renvoy une liste vide !");
		return;
	}

	my $target = $path.$ra_hash_dir->[0];
	my $found;
	foreach my $dir (@{$ra_hash_dir}) {
		my $p = $path.$dir;
		# si le nom est deja utilise dans l'arbo
		if ( -e $p."/".$name ) {
			__debug "path trouve : ".$p;
			$found = $p;
			last;
		}
	}

	__debug "File not found for session $name" if ! $found;
	return ( $found, $target ) if wantarray;
	return $target if ! $found;
	return $found;
}


my $rand_function;
sub __random {
	return &$rand_function() if defined $rand_function;
	if ( conduse('Crypt::Random') ) {
		$rand_function = sub () { return Crypt::Random::makerandom(Size => 32); };
		return &$rand_function();
	} else {
		srand (time ^ $$ ^ unpack "%L*", `ps axww | gzip -f`);
		$rand_function = sub () { return rand(); };
		return &$rand_function();
	}
}

# ----------------------------------------------------------------------
# name   : __gennomsession 
# ----------------------------------------------------------------------
sub __gennomsession ($$$$) {
	my ( $path, $prefix, $hash_type, $hash_depth) = @_;
	
	if ( !defined($prefix) ) {
		error "prfixe non defini";
		return;
	}
	
	if ( !defined($path) ) {
		error "path non defini";
		return;
	}
	
	if ( !defined($hash_depth) ) {
		error "depth non defini";
		return;
	}
	
	if ( !defined($hash_type) ) {
		error "hash_type non defini";
		return;
	}
	
	my ($check, $nom) = (0, '');
	my $npath = $path;
	do {
		$check = 1;
		my $rand = __random();
		$nom = $prefix . time() . '_' . $rand;
		# genreation du hash de stockage en fonction du nom genere par rand
		my $ra_hash_dir = safe_dir_hashing( $hash_type, $nom, $hash_depth, 0);
		last if ! defined $ra_hash_dir;
		unless (scalar @{$ra_hash_dir}) {
			error("il faut utiliser __init_conf avant __gennomsession");
			last;
		}

		my $dir;
		foreach $dir (@$ra_hash_dir) {
			$npath = $path . $dir;
			__debug "Checking existence en previous file in $npath/$nom";
			# si le nom est deja utilise dans l'arbo
			if (-e $npath."/".$nom) {
				__debug "Found it, need another name";
				$check = 0;
				last;
			}
		}
		$dir = $ra_hash_dir->[0];
		$npath = $path.$dir;
	} while ( !$check );

	__debug 'nom genere : '.$npath."/".$nom;
	return ($nom, $npath);
}

#############################################################################
#
#   Gestion des verrous
#
#############################################################################

my $locks = {};

sub __prepare_lock($$) {
	my ( $type, $conf ) = @_;
	return if ! $conf->{'use_lock'};
	conduse('FDN::Common::FileLock');
	$locks->{$type} = {} if ! exists $locks->{'type'};
}

sub unlock_session($) {
	my ( $sess ) = @_;

	my $name = $sess->{'name'};
	my $type = $sess->{'type'};
	if ( exists $locks->{$type}->{$name} ) {
		my $FH = $locks->{$type}->{$name};
		unsetlock($FH);
		close $FH;
		delete $locks->{$type}->{$name};
	}
	return TRUE;
}


#############################################################################
#
#   Gestion des fichiers
#
#############################################################################

sub __check_session($$) {
	my ( $name, $conf ) = @_;

	my $check;

	my ( $path, $target ) = __find_path(
		$name,
		$conf->{'path'},
		$conf->{'hash_type'},
		$conf->{'hash_depth'},
		$conf->{'hash_fallback'});
	my $file;

	if ( $path ) {
		$file = $target.'/'.$name;
		if ( $path ne $target ) {
			__debug "Found $name in the wrong place, moving from $path to $target";
			rename $path.'/'.$name, $file;
		}
		$check = age_bon ( $file, $conf->{'time_out'} );
		if ( ! $check ) {
			__debug "Found $name to be outdated";
			unlink $file;
			$file = undef;
		}
	}

	return $file;
}

sub __open_session($$;$) {
	my ( $name, $conf, $can_create ) = @_;

	my $file = __check_session($name, $conf);
	my $FH;

	if ( ! $file ) {
		return undef if ! $can_create;
	}

	if ( ! open ( $FH, '+<', $file ) ) {
		__sessalert( $conf->{'mail_warn'},
			"__open_session : problme d'ouverture du fichier de session $file : $!",
			$conf->{'mail_list'});
		return undef;
	}

	return $FH;
}

sub __read_session($$) {
	my ( $FH, $conf ) = @_;
	my $session;

	seek($FH,0,0);
	eval { $session = fd_retrieve ( $FH ); };
	if ( $@ ) {
		__debug "Failed to read the session file...";
		__sessalert($conf->{'mail_warn'},
			"retrieve_session : problme de recup de contenu de ".
			"session ($@) !",
			$conf->{'mail_list'});
		return;
	}
	if ( ref ( $session->{'content'} ) eq 'HASH' ) {
		__debug "retrieve_session : On a retrouv la session";
#		if ( $session->{'type'} ne $type ) {
#			error "Expected type $type got ".$session->{'type'}.
#			      " I'm puzzled.";
#		}
#		if ( $session->{'name'} ne $name ) {
#			error "Expected name $name got ".$session->{'name'}.
#			      " I'm puzzled.";
#		}
		return $session;
	}
	# On ne devrait jamais arriver dans ce cas la !
	__sessalert( $conf->{'mail_warn'},
		"retrieve_session : problme de recup de contenu de ".
		"session (session corrompue) !",
		$conf->{'mail_list'});
	return;
}



#############################################################################
#
#   Gestion des sessions
#
#############################################################################

#############################################################################
#
#   Fonctions exportes
#
#############################################################################

# ----------------------------------------------------------------------
# name   : retrieve_session 
# params : $type    : le type de session
#          $name    : le nom du fichier
#	   $do_lock : faut-il garder un verrou ?
# return : une HREF sur {
#              name => le nom du fichier genene
#              type => le type de la session
#              content => {...}
#          }
#          undef si un pobleme
# ----------------------------------------------------------------------
sub retrieve_session ($$;$)  {
	my ( $type, $name, $do_lock ) =  @_;
	my $session;
	
	$do_lock = 0 if ! defined $do_lock;
	if ( ! defined $name || ! defined $type ) {
		error "Wrong parameters";
		return;
	}

	my $conf = __init_conf($type);
	$debug = $conf->{'sess_debug'};
	__debug "Retrieving session with name=".$name;

	my $FH;
	if ( $conf->{'use_lock'} && exists $locks->{$type}->{$name} ) {
		__debug "The session file is already opened, and we have a lock on it, easy doing then.";
		$FH = $locks->{$type}->{$name};
		return __read_session($FH,$conf);
	}
	
	__debug "Need to find, and open, the session file";
	$FH = __open_session($name,$conf,0);
	return if ! $FH;
	if ( $conf->{'use_lock'} ) {
		__debug "Getting a lock on the file";
		__prepare_lock($type,$conf);
		setlock($FH,1);
		if ( $do_lock ) {
			__debug "Saving the lock since do_lock=$do_lock and we are using locks on those sessions";
			$locks->{$type}->{$name} = $FH;
		}
	}
	$session = __read_session($FH, $conf);
	if ( ! exists $locks->{$type}->{$name} ) {
		if ( $conf->{'use_lock'} ) {
			__debug "Releasing the lock on $name, since I did not save it";
			unsetlock($FH);
		}
		close $FH;
	}
	return $session;
}


# ----------------------------------------------------------------------
# name   : refresh_session
# desc   : simule un touch sur un fichier
# params : $type : le type de session
#          $name : le nom du fichier
# return : 1 si tout va bien
#          undef sinon
# ----------------------------------------------------------------------
sub refresh_session ($$) {
	my ( $type, $name ) =  @_;
	
	if ( ! defined $type || !defined $name ) {
		error "Wrong parameters";
		return FALSE;
	}
	
	my $conf = __init_conf($type);
	if ( ! defined $conf ) {
		error "Undefined session type";
		return FALSE;
	}
	$debug = $conf->{'sess_debug'};

	my $file = __check_session($name, $conf);

	if ( ! defined $file) {
		__debug "Did not find the session file for $name on disk. Probably an expired session. Cannot refresh.";
		return FALSE;
	}

	utime undef, undef, $file;
	__debug "refresh ".$file;
	return TRUE;
}


# ----------------------------------------------------------------------
# name   : new_session 
# desc   : cree un fichier de session sur le disque
# params : $type : le type de la session 
# return : une HREF sur {
#              name => le nom du fichier genene
#              type => le type de la session
#              content => {}
#          }
#          undef si un pobleme
# ----------------------------------------------------------------------
sub new_session ($) {
	my ( $type ) = @_;
	my $conf = __init_conf($type);
	$debug = $conf->{'sess_debug'};
	
	my ($name, $path) = __gennomsession(
				$conf->{'path'},
				$conf->{'prefix'},
				$conf->{'hash_type'},
				$conf->{'hash_depth'});

	unless ($name and $path) {
		error "Impossible de gnrer un nom de session";
		return;
	}
	__debug "new session: name=$name, path=$path";

	if ( ! -d $path ) {
		__debug "Need to create the directory";
		if ( !mkdir_p($path) ) {
			error("echec creation du repertoire $path");
			return;
		}
	}
	__cleaner ($path, $conf->{'time_out'}, $conf->{'prefix'},
		   $conf->{'hash_fallback'}) if $conf->{'auto_wash'};
	if ( !open SESSFILE,">".$path."/".$name ) {
		error("Impossible de creer le fichier de session ".$path."/".$name.": ".$!);
		return;
	}

	debug "Created session in file $path/$name";

	my $session = {
		type => $type,
		name => $name,
		content => {}
	};
	eval { store_fd( $session, *SESSFILE); };
	if ($@) {
		__sessalert($conf->{'mail_warn'},
			"newwsession : problme d'criture dans $path/$name (store_fd: $@)",
			$conf->{'mail_list'});
		return;
	}
	
	unless (close SESSFILE) {
		__sessalert($conf->{'mail_warn'},
			"newwsession : problme d'criture dans $path/$name (close: $!)",
			$conf->{'mail_list'});
		return;
	};

	return $session;
}


# ----------------------------------------------------------------------
# name   : store_session
# ----------------------------------------------------------------------
sub store_session ($) {
	my ( $session ) =  @_;
	
	if ( ref($session) ne 'HASH' ) {
		error "Wrong parameters";
		return;
	}

	my $type = $session->{'type'};
	if ( ! defined $type ) {
		error "Missing type";
		return;
	}

	my $name = $session->{'name'};
	if ( ! defined $name ) {
		error "Missing name";
		return;
	}
	
	# on charge la conf
	my $conf = __init_conf($type);
	if ( ! defined $conf ) {
		error "Wrong session type";
		return;
	}
	$debug = $conf->{'sess_debug'};
	
	__debug "Saving session with name=$name";


	if ( $conf->{'use_lock'} ) {
		my $FH;
		__debug "We are in lock mode for type=$type";
		if ( exists $locks->{$type}->{$name} ) {
			__debug "Session is already locked";
			$FH = $locks->{$type}->{$name};
			delete $locks->{$type}->{$name};
		} else {
			__debug "Getting the lock, since not already locked";
			$FH = __open_session($name, $conf, 1);
			__prepare_lock($type,$conf);
			setlock($FH,1);
		}
		seek($FH,0,0);
		truncate($FH,0);
		eval { store_fd ( $session , $FH ) };
		close($FH);
		unsetlock($FH);
		return $name;
	}

	my $path = __find_path(
		$name,
		$conf->{'path'},
		$conf->{'hash_type'},
		$conf->{'hash_depth'},
		$conf->{'hash_fallback'});
	
	__cleaner ($path, $conf->{'time_out'}, $conf->{'prefix'},
		   $conf->{'hash_fallback'}) if $conf->{'auto_wash'};
	
	my $filename = $path . "/" . $name;
	__debug "filename=$filename";

	my $aux_filename = "$filename.$hostname.$$";
	__debug "aux_filename=$aux_filename";
	
	unless ( open ( SESSFILE , ">".$aux_filename ) ) {
		__sessalert( $conf->{'mail_warn'},
			"store_session : problme d'ouverture du fichier de session :".
			" $aux_filename ($!)",
			$conf->{'mail_list'} );
		return;
	}
	eval { store_fd ( $session , *SESSFILE ); };
	if ($@) {
		__sessalert($conf->{'mail_warn'},
			"store_session : problme d'criture dans $aux_filename ".
			"(store_fd: $@)",
			$conf->{'mail_list'});
		close SESSFILE;
		return;
	}
	unless (close SESSFILE) {
		__sessalert($conf->{'mail_warn'},
			"store_session : problme d'criture dans $aux_filename ".
			"(close: $!)",
			$conf->{'mail_list'});
		return;
	}

	__debug "on renomme le ficher $aux_filename en $filename";

	unless (rename $aux_filename, $filename) {
		__sessalert($conf->{'mail_warn'},
			"store_session : problme pour renomer $aux_filename en $filename",
			$conf->{'mail_list'});
		return;
	}

	__debug "session enregistree : " . $name;
	return $name;
}


# ----------------------------------------------------------------------
# name   : close_session 
# ----------------------------------------------------------------------
sub close_session ($$) {
	my ( $type, $name ) =  @_;
	
	if ( ! defined $type || ! defined $name ) {
		error "Wrong parameters";
		return FALSE;
	}
	
	my $conf = __init_conf($type);
	if ( ! defined $type ) {
		error "Undefined session type ($type)";
		return FALSE;
	}
	

	my $path = __find_path( $name,
		$conf->{'path'},
		$conf->{'hash_type'},
		$conf->{'hash_depth'},
		$conf->{'hash_fallback'});
	my $file =  $path."/".$name;
	if ( -e $file ){
		__debug ' fermeture de '. $file;
		my $ret = unlink $file;
		if ( $conf->{'use_lock'} && exists $locks->{$type}->{$name} ) {
			my $FH = $locks->{$type}->{$name};
			unsetlock($FH);
			close $FH;
			delete $locks->{$type}->{$name};
		}
		return $ret;
	} else {
		return TRUE;
	}
}


# ----------------------------------------------------------------------
# name        : age_bon 
# description : vrifie l'ge d'un fichier 
# parametres  : filename = nom du fichier  tester
#               peremption = dure de validit du fichier
# return      : 1 si la diffrence entre la date et mtime ou atime du 
#                 fichier est < peremtion
#               0 dans le cas contraire
#		undef si le fichier n'existe pas
# ----------------------------------------------------------------------
sub age_bon ($$) {
        my ($filename, $peremption) = @_;

        my $dt = stat $filename;
	return FALSE if ( ! $dt );

	unless ($dt->mtime || $dt->atime) {
		error("pas de date");
		return FALSE;
	}

#if ( use_lock && locked ) {
#	touch;
#	return TRUE;
#}

	my $delta = $dt->mtime > $dt->atime ? $dt->mtime : $dt->atime;
	my $age = time - $delta; 

	if ($age < $peremption) {
		__debug "age_bon oui ($age, $delta, ".$dt->mtime.", ".$dt->atime.")";
		return TRUE;
	}

	__debug "age_bon non : $age, peremption : $peremption, atime : ".$dt->atime.
		", mtime: ".$dt->mtime;
	return FALSE;
}



#
#  N'appelle dir_hashing() que si $depth est > 0 !
#
sub safe_dir_hashing ($$$$) {
	my ($type, $name, $depth, $fallback) = @_;

	my $ra_hash_dir = [];
	
	if ($depth) {
	    $ra_hash_dir = dir_hashing($type, $name, $depth, $fallback);
	}

	push @{$ra_hash_dir}, '';	# On veut toujours avoir le rpertoire racine
					# (pour compatibilit en lecture et nettoyage)

	return $ra_hash_dir;
}


1;

